@@ -42,12 +42,6 @@
 #include <sys/socket.h>
 #include <ifaddrs.h>
 
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
-#define HAS_PANEL_49
-#endif
-#endif
-
 #define BORDER 8
 typedef struct
 {
@@ -81,7 +75,11 @@ static void wavelan_set_orientation(XfcePanelPlugin* plugin, GtkOrientation orie
 static void
 wavelan_set_state(t_wavelan *wavelan, gint state)
 {  
-  GdkColor color;
+  GdkRGBA color;
+#if GTK_CHECK_VERSION (3, 16, 0)
+  GtkCssProvider *css_provider;
+  gchar *css;
+#endif
 
   gchar signal_color_bad[] = "#e00000";
   gchar signal_color_weak[] = "#e05200";
@@ -102,23 +100,39 @@ wavelan_set_state(t_wavelan *wavelan, gint state)
    if (wavelan->signal_colors) {
     /* set color */
      if (state > 70)
-      gdk_color_parse(signal_color_strong, &color);
+      gdk_rgba_parse(&color, signal_color_strong);
      else if (state > 55)
-      gdk_color_parse(signal_color_good, &color);
+      gdk_rgba_parse(&color, signal_color_good);
      else if (state > 40)
-      gdk_color_parse(signal_color_weak, &color);
+      gdk_rgba_parse(&color, signal_color_weak);
      else
-      gdk_color_parse(signal_color_bad, &color);
+      gdk_rgba_parse(&color, signal_color_bad);
 
-        gtk_widget_modify_bg(GTK_WIDGET(wavelan->signal),
+#if GTK_CHECK_VERSION (3, 16, 0)
+#if GTK_CHECK_VERSION (3, 20, 0)
+     css = g_strdup_printf("progressbar progress { background-color: %s; background-image: none; }", gdk_rgba_to_string(&color));
+#else
+     css = g_strdup_printf(".progressbar { background-color: %s; background-image: none; }", gdk_rgba_to_string(&color));
+#endif
+     /* Setup Gtk style */
+     css_provider = gtk_css_provider_new ();
+     gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
+     gtk_style_context_add_provider (
+         GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (wavelan->signal))),
+         GTK_STYLE_PROVIDER (css_provider),
+         GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+     g_free(css);
+#else
+     gtk_widget_override_background_color(GTK_WIDGET(wavelan->signal),
                              GTK_STATE_PRELIGHT,
                              &color);
-        gtk_widget_modify_bg(GTK_WIDGET(wavelan->signal),
+     gtk_widget_override_background_color(GTK_WIDGET(wavelan->signal),
                              GTK_STATE_SELECTED,
                              &color);
-        gtk_widget_modify_base(GTK_WIDGET(wavelan->signal),
+     gtk_widget_override_color(GTK_WIDGET(wavelan->signal),
                              GTK_STATE_SELECTED,
                              &color);
+#endif
     }
 
    }
@@ -317,7 +331,7 @@ wavelan_new(XfcePanelPlugin *plugin)
   g_object_ref( wavelan->tooltip_text );
 
   /* create box for img & progress bar */
-  wavelan->box = xfce_hvbox_new(wavelan->orientation, FALSE, 0);
+  wavelan->box = gtk_box_new(wavelan->orientation, 0);
 
   /* setup progressbar */
   wavelan->signal = gtk_progress_bar_new();
@@ -398,14 +412,14 @@ wavelan_set_orientation(XfcePanelPlugin* plugin, GtkOrientation orientation, t_w
 {
   DBG("wavelan_set_orientation(%d)", orientation);
   wavelan->orientation = orientation;
-  xfce_hvbox_set_orientation(XFCE_HVBOX(wavelan->box), orientation);
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(wavelan->box), orientation);
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(wavelan->signal), !orientation);
+  gtk_progress_bar_set_inverted(GTK_PROGRESS_BAR(wavelan->signal), (orientation == GTK_ORIENTATION_HORIZONTAL));
   if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-   gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(wavelan->signal), GTK_PROGRESS_BOTTOM_TO_TOP);
    gtk_widget_set_size_request(wavelan->signal, 8, -1);
    gtk_widget_set_size_request(wavelan->ebox, -1, wavelan->size);
    }
   else {
-   gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(wavelan->signal), GTK_PROGRESS_LEFT_TO_RIGHT);
    gtk_widget_set_size_request(wavelan->signal, -1, 8);
    gtk_widget_set_size_request(wavelan->ebox, wavelan->size, -1);
    }
@@ -416,10 +430,8 @@ wavelan_set_size(XfcePanelPlugin* plugin, int size, t_wavelan *wavelan)
 {
   int border_width, image_size;
   DBG("wavelan_set_size(%d)", size);
-#ifdef HAS_PANEL_49
   size /= xfce_panel_plugin_get_nrows(plugin);
   xfce_panel_plugin_set_small (plugin, TRUE);
-#endif
   border_width = size > 26 ? 2 : 1;
   wavelan->size = size;
   image_size = wavelan->size - (2 * border_width);
@@ -499,9 +511,9 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   TRACE ("Entered wavelan_create_options");
   
   dlg = xfce_titled_dialog_new_with_buttons (_("Wavelan Plugin Options"),
-              NULL,
-              GTK_DIALOG_NO_SEPARATOR,
-              GTK_STOCK_CLOSE,
+              GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+              GTK_DIALOG_DESTROY_WITH_PARENT,
+              "gtk-close",
               GTK_RESPONSE_OK,
               NULL);
 
@@ -515,13 +527,13 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 
   xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dlg), _("Properties"));
               
-  vbox = gtk_vbox_new(FALSE, 8);
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_widget_show(vbox);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), vbox,
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dlg))), vbox,
                       TRUE, TRUE, 0);
   
-  hbox = gtk_hbox_new(FALSE, 8);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
   gtk_widget_show(hbox);
   
   label = gtk_label_new(_("Interface"));
@@ -544,7 +556,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, FALSE, 1);
   gtk_box_pack_start(GTK_BOX(hbox), combo, TRUE, FALSE, 1);
 
-  hbox = gtk_hbox_new(FALSE, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_widget_show(hbox);
   autohide = gtk_check_button_new_with_mnemonic(_("_Autohide when offline"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide), wavelan->autohide);
@@ -554,7 +566,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), autohide, TRUE, TRUE, 1);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
   
-  hbox = gtk_hbox_new(FALSE, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_widget_show(hbox);
   autohide_missing = gtk_check_button_new_with_mnemonic(_("Autohide when no _hardware present"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide_missing), 
@@ -565,7 +577,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), autohide_missing, TRUE, TRUE, 1);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
 
-  hbox = gtk_hbox_new(FALSE, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_widget_show(hbox);
   warn_label = gtk_label_new(_("Note: This will make it difficult to remove or configure the plugin if there is no device detected."));
   gtk_label_set_line_wrap(GTK_LABEL(warn_label), TRUE);
@@ -573,7 +585,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), warn_label, TRUE, TRUE, 1);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
 
-  hbox = gtk_hbox_new(FALSE, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_widget_show(hbox);
   signal_colors = gtk_check_button_new_with_mnemonic(_("Enable sig_nal quality colors"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(signal_colors), 
@@ -584,7 +596,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), signal_colors, TRUE, TRUE, 1);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
 
-  hbox = gtk_hbox_new(FALSE, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_widget_show(hbox);
   show_icon = gtk_check_button_new_with_mnemonic(_("Show _icon"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_icon), 
